#ifndef __DBGFLOPPY_POLL_MODE__H__
#define __DBGFLOPPY_POLL_MODE__H__

extern "C" {

#include <ntddk.h>
#include <ntdddisk.h>

};
#include "stddef.h"

typedef struct _FLOPPY_PORT_LAYOUT {
    UCHAR Reserved0; // PS/2, Intel only    3f0
    UCHAR MediaId;   // PS/2, Intel only    3f1
    UCHAR DriveControl;
    UCHAR Reserved1;
    UCHAR Status;
    UCHAR Fifo;
    UCHAR Reserved2;
    union {
        UCHAR DataRate;
        UCHAR DiskChange;
    } DRDC;
} FLOPPY_PORT_LAYOUT, *PFLOPPY_PORT_LAYOUT;

//
// Floppy commands.                                Optional bits allowed.
//

#define COMMND_READ_DATA                   0x06    // Multi-Track, MFM, Skip
#define COMMND_READ_DELETED_DATA           0x0C    // Multi-Track, MFM, Skip
#define COMMND_READ_TRACK                  0x02    // MFM
#define COMMND_WRITE_DATA                  0x05    // Multi-Track, MFM
#define COMMND_WRITE_DELETED_DATA          0x09    // Multi-Track, MFM
#define COMMND_READ_ID                     0x0A    // MFM
#define COMMND_FORMAT_TRACK                0x0D    // MFM
#define COMMND_RECALIBRATE                 0x07
#define COMMND_SENSE_INTERRUPT             0x08
#define COMMND_SPECIFY                     0x03
#define COMMND_SENSE_DRIVE                 0x04
#define COMMND_SEEK                        0x0F
#define COMMND_PERPENDICULAR_MODE          0x12
#define COMMND_CONFIGURE                   0x13

//
// Optional bits used with the commands.
//

#define COMMND_MULTI_TRACK                 0x80
#define COMMND_MFM                         0x40
#define COMMND_SKIP                        0x20


//
// Bits in the DRIVE_CONTROL register.
//

#define DRVCTL_RESET                       0x00
#define DRVCTL_ENABLE_CONTROLLER           0x04
#define DRVCTL_ENABLE_DMA_AND_INTERRUPTS   0x08
#define DRVCTL_DRIVE_0                     0x10
#define DRVCTL_DRIVE_1                     0x21
#define DRVCTL_DRIVE_2                     0x42
#define DRVCTL_DRIVE_3                     0x83
#define DRVCTL_DRIVE_MASK                  0x03
#define DRVCTL_MOTOR_MASK                  0xf0

//
// Bits in the STATUS register.
//

#define STATUS_DRIVE_0_BUSY                0x01
#define STATUS_DRIVE_1_BUSY                0x02
#define STATUS_DRIVE_2_BUSY                0x04
#define STATUS_DRIVE_3_BUSY                0x08
#define STATUS_CONTROLLER_BUSY             0x10
#define STATUS_DMA_UNUSED                  0x20
#define STATUS_DIRECTION_READ              0x40
#define STATUS_DATA_REQUEST                0x80

#define STATUS_IO_READY_MASK               0xc0
#define STATUS_READ_READY                  0xc0
#define STATUS_WRITE_READY                 0x80

//
// Bits in the DATA_RATE register.
//

#define DATART_0125                        0x03
#define DATART_0250                        0x02
#define DATART_0300                        0x01
#define DATART_0500                        0x00
#define DATART_1000                        0x03
#define DATART_RESERVED                    0xfc

//
// Bits in the DISK_CHANGE register.
//

#define DSKCHG_RESERVED                    0x7f
#define DSKCHG_DISKETTE_REMOVED            0x80

typedef struct _FLOPPYDRV {
    DISK_GEOMETRY Geom;
    PFLOPPY_PORT_LAYOUT Port;            // 3f0
    UCHAR               FifoBuffer[10];
    ULONG DriveId; //0-3
    ULONG DriveType;
    ULONG CurPos;
    ULONG Size;
} FLOPPYDRV, *PFLOPPYDRV;

#endif //__DBGFLOPPY_POLL_MODE__H__
